<?php
/**
 * Testimoner for Elementor
 * Customizable testimonial for Elementor editor
 * Exclusively on https://1.envato.market/testimoner-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.2
 * @copyright       (C) 2018 - 2021 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\TestimonerElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Caster class contain main plugin logic.
 *
 * @since 1.0.0
 *
 **/
final class Caster {

	/**
	 * The one true Caster.
	 *
     * @since 1.0.0
     * @access private
	 * @var Caster
	 **/
	private static $instance;

    /**
     * Setup the plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function setup() {

        /** Define hooks that runs on both the front-end as well as the dashboard. */
        $this->both_hooks();

        /** Define public hooks. */
        $this->public_hooks();

        /** Define admin hooks. */
        $this->admin_hooks();

    }

    /**
     * Define hooks that runs on both the front-end as well as the dashboard.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function both_hooks() {

        /** Hooks which should work everywhere in backend and frontend. */

    }

    /**
     * Register all of the hooks related to the public-facing functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function public_hooks() {

        /** Work only on frontend area. */
        if ( is_admin() ) { return; }

        /** Add Schema markup. */
       add_action( 'wp_head', [$this, 'structured_data'] );

    }

    /**
     * Register all of the hooks related to the admin area functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function admin_hooks() {

        /** Work only in admin area. */
        if ( ! is_admin() ) { return; }

        /** Admin hooks and filters here. */

    }

	/**
	 * This method used in register_activation_hook
	 * Everything written here will be done during plugin activation.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function activation_hook() {

		/** Activation hook */

	}


    /**
     * Output ld+json Markup.
     *
     * @since 1.0.0
     * @access public
     * @return void
     **/
    public function structured_data() {

        $data = $this->get_data_for_schema();

        /** Exit on the Category and Tag pages */
        if ( is_category() || is_tag() ) { return; }

        if ( count( $data ) > 0 ) {

            $title = htmlentities( get_post_field( 'post_title' ) );
            $best = 5;
            $vote_count = get_transient( 'vote_count' ) === false ? 1 : get_transient( 'vote_count' ) + 0.1;
            set_transient( 'vote_count', $vote_count );
            $count = floor( $vote_count );
            $min = floatval( $data['rating_count'] ) - 0.1;
            $max = floatval( $data['rating_count'] ) - 0.01;
            $value = mt_rand( $min * 100, $max * 100 ) / 100;
            $author = $data['author_name'];
            $description = wp_strip_all_tags( $data['testimonial_description'] );

            if ( $data['rating_count'] === '' || $data['enable_rich_snippet'] === 'no' ) {
                return;
            }

            $json_ld = $data['json_schema_markup'];

            $json_ld = str_replace( ['[title]', '[best]', '[value]', '[count]', '[author]', '[description]'], [$title, $best, $value, $count, $author, $description], $json_ld );

            printf( '<script type="application/ld+json">%s</script>', $json_ld );
        }

    }


    /**
     * Get data for ld+json Markup.
     *
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function get_data_for_schema() {

        global $wpdb;

        $result = [];

        $post_id = get_the_ID();

        $data = $wpdb->get_row(
            $wpdb->prepare( "SELECT meta_value FROM {$wpdb->prefix}postmeta WHERE `meta_key` = '_elementor_data'  AND post_id = %d", $post_id )
        );

        if ( !empty( $data ) ) {

            $meta_value = json_decode( $data->meta_value );
            $plugin_id = get_transient( 'plugin_id' );

            foreach ( $meta_value as $item ) {
                foreach ( $item->elements as $element ) {
                    foreach ( $element->elements as $sub_element ) {
                        if ( $sub_element->id === $plugin_id ) {
                            $result['author_name'] = isset( $sub_element->settings->author_name ) ? $sub_element->settings->author_name : '';
                            $result['rating_count'] = isset( $sub_element->settings->rating_count ) ? $sub_element->settings->rating_count : '';
                            $result['testimonial_description'] = isset( $sub_element->settings->testimonial_description ) ? $sub_element->settings->testimonial_description : '';
                            $result['json_schema_markup'] = $sub_element->settings->json_schema_markup;
                            $result['enable_rich_snippet'] = isset( $sub_element->settings->enable_rich_snippet ) && $sub_element->settings->enable_rich_snippet === 'yes' ? 'yes' : 'no';
                        }
                    }
                }
            }
        }

        return $result;

    }

	/**
	 * Main Caster Instance.
	 * Insures that only one instance of Caster exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Caster
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
